/*
 * Eufruncio - El gusanito comefichas
 * Copyright (C) 2003-2004 Sebastin Galiano, Milton Pividori, Csar Sandrigo
 * 
 * This file is part of Eufruncio.
 * 
 * Eufruncio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Eufruncio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Eufruncio; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/************************************************************************************************
	 Definimos los tipos que utilizaremos
************************************************************************************************/

/* Estructura que representa cada segmento del cuerpo del gusano */
typedef struct gusano
{
  char fila;
  char columna;
  struct gusano *siguiente;
}gusano;



/************************************************************************************************
	 Reserva dinmicamente memoria para almacenar un nuevo nodo del gusano
************************************************************************************************/

gusano *NuevoNodo()
{
   gusano *p;
   p = (gusano *) malloc(sizeof(gusano));
   
   return p;
}



/************************************************************************************************
	Liberamos de la memoria la lista que representa al gusano
************************************************************************************************/

void LiberarGusano(gusano **cabeza)
{
	gusano *aux1 = *cabeza;
	gusano *temp1;

	while (aux1 != NULL)
	{
		temp1 = aux1;
		aux1 = aux1->siguiente;

		free(temp1);
	}

	*cabeza = aux1 = NULL;
}



/************************************************************************************************
	Borramos en la pila la ficha correspondiente 
************************************************************************************************/

void EliminarPrimerFicha(ficha **fichas)
{
   ficha *aux = *fichas;

	*fichas = aux->siguiente;
	free(aux);
}



/************************************************************************************************
	 Dibuja las paredes
************************************************************************************************/
	
void DibujarParedes(int Color)
	{ 		 
		char i;
	
		textcolor(Color);
		for (i=1; i<ANCHO; i++) 					/* Dibujamos las dos paredes horizontales */				
			{
				gotoxy(i,2); cprintf("%c", 219);
				gotoxy(i,ALTO-2); cprintf("%c", 219);
			} 		 
		for(i=2; i<ALTO-1; i++) 							/* Dibujamos las dos paredes verticales */ 
			{
				gotoxy(1,i); cprintf("%c", 219);
				gotoxy(ANCHO, i); cprintf("%c", 219);
			}
	}



/************************************************************************************************
	 Dibuja los obstculos
************************************************************************************************/

void VisualizarObstaculos(obs *obstaculos[])
{
	 obs *aux_obs;
	 char i=0;
     
	 textcolor(COLOROBSTACULO);
     
	 while (i<=44)
	 {
			aux_obs = obstaculos[i];
			while (aux_obs != NULL)
			{
				 /* Se le suma tres a la fila por el espacio en blanco superior, por
				 	 el borde, y porque obstaculos es un vector que comienza con el
				 	 0 */
				 gotoxy(aux_obs->columna + 1, i + 3);
				 cprintf("O");

				 aux_obs = aux_obs->siguiente;
			}
      
      i++;
   }
}



/************************************************************************************************
	 Carga al gusano en memoria y lo muestra en pantalla.
************************************************************************************************/

void CrearGusano(gusano **p)
{      
   int i;
   gusano *temporal;
   gusano *cabeza = *p;

   textcolor(COLORGUSANO);
   
   for (i=0; i<=LONGITUDINICIAL; i++)
   {
      temporal = NuevoNodo();
		temporal->columna = 40;
      temporal->fila = 27-i;
		temporal->siguiente = cabeza;
      gotoxy(temporal->columna, temporal->fila);
      cprintf("%c", 219);
      
		cabeza = temporal;
   }
   
   *p = cabeza;
}



/************************************************************************************************
	 Mostramos en pantalla una ficha
************************************************************************************************/

void VisualizarFicha(char Columna, char Fila)
{
  textcolor(COLORFICHA);
  gotoxy(Columna + 1, Fila + 2);
  cprintf("F");
}



/************************************************************************************************
	 Mostramos en pantalla la siguiente ficha
************************************************************************************************/

char MostrarSiguienteFicha(ficha *fichas, gusano *cabeza)
{
	gusano *aux=cabeza;	
	
    if (fichas == NULL)
		return 0;

	while ((aux != NULL) && ((aux->columna != (fichas->columna + 1)) || (aux->fila != (fichas->fila + 2))))
		aux = aux->siguiente;

	if (aux == NULL)
	{
	   	VisualizarFicha(fichas->columna, fichas->fila);
		return 0;
	}
	else
		return 1;
}



/************************************************************************************************
	Mostramos el puntaje y otros datos del juego
************************************************************************************************/

void Datos(datos Gusano)
{
    /* Declaraciones */
    char Tecla, i, j;

    /* Configuramos los colores */
    textbackground(COLORFONDO); textcolor(COLORBARRAESTADO);

    /* Mostramos el puntaje */
    gotoxy(3,49);
    cprintf("Puntaje: %ld", Gusano.puntaje);

    /* Mostramos las fichas recogidas */
    gotoxy(60,49);
    cprintf("Fichas recogidas: %u", Gusano.fichas_comidas);
}



/************************************************************************************************
	 Se fija lo que hay en la pantalla en la posicin indicada
************************************************************************************************/

/* Esta funcion devuelve un 0 si no hay nada, un 1 si hay un obstaculo, y un 2 si hay un ficha */
char QueHay(obs *obstaculos[], ficha *fichas, gusano *cabeza, char *FichaSobreGusano, char columna, char fila)
{
	 obs *aux_obs;
	 ficha *aux_ficha;
	 gusano *aux_gusano=cabeza->siguiente;

	 if (*FichaSobreGusano == 1)
	 {
		*FichaSobreGusano = MostrarSiguienteFicha(fichas, cabeza);
	 }

	 /* Verificamos que no choque el borde */
	 if ((columna == 1) || (columna == 80))
			return 1;

	 if ((fila == 2) || (fila == 48))
			return 1;

	 while ((aux_gusano != NULL) && ((aux_gusano->columna != columna) || (aux_gusano->fila != fila)))
		aux_gusano = aux_gusano->siguiente;

	 if (aux_gusano != NULL)
		return 1;

	 /* Verificamos si hay un obstculo */
	 /* Le restamos 3 porque fila esta incluyendo el borde y el espacio superior,
	 	 y adems, el vector obstaculos comienza con el 0. Es decir que la
	 	 informacion de la fila 1 esta en la componente 0 */
	 aux_obs = obstaculos[fila - 3];
	 
	 while ((aux_obs != NULL) && (aux_obs->columna  != (columna - 1)))
			aux_obs = aux_obs->siguiente;

	 if (aux_obs != NULL)
			return 1;

	 /* Verificamos si hay una ficha */
	 if ((fichas->columna == (columna - 1)) && (fichas->fila == (fila - 2)))
			return 2;

	 return 0;
}


/************************************************************************************************
	 Mueve al gusano en la pantalla
************************************************************************************************/

/* La funcin devuelve 1 si choca con un obstculo, con ella misma o con las paredes; y 2 cuando
   se comieron todas fichas. Devuelve 0 si no se cuimple nada de lo anterior */
char MoverGusano(gusano **p, char giro, datos *Eufruncio, obs *obstaculos[], ficha **fichas)
{
   typedef struct aux
   {
		gusano *info;
      struct aux *siguiente;
	}aux;
   
   /* Si la siguiente variable esttica es 0, es porque el gusano no debe
      crecer, pero si es uno, debe ir creciendo porque hay comido una ficha.
      Creciendo ser nuevamente 0 cuando VecesQueDebeCrecer llegue a valer 0. */
   static char Creciendo = 0;
   static char VecesQueDebeCrecer = 0;
      
   gusano *cabeza = *p;
   gusano *temp; /* Puntero temporal */
   
   /* Puntero al inicio de una lista auxiliar con nodos del tipo struct aux */
   aux *lista_aux=NULL;
   aux *temp2;

	/* Al chequear la direccin del gusano, calculamos los aumentos en la
	   posicion de la cabez */
	char aumento_fila, aumento_col;

	/* Para almacenar el resultado de la funcion QueHay y as no llamarla varias
		veces */
	char Resultado_QueHay;

	/* Esto sirve para cuando la siguiente ficha a dibujar esta sobre el
	   gusano */
	static char FichaSobreGusano=0;
	
	datos *Gusano = Eufruncio;

	textcolor(COLORGUSANO);
 
    /* Si *fichas es NULL, no hay mas fichas, y el juego se termina */
	if (*fichas == NULL)
	{
		Creciendo = 0;
		VecesQueDebeCrecer = 0;

		return 2;
	}

	if (giro == ' ')
	{
		/* Nos fijamos la direccion del gusano... */
		switch (Gusano->direccion)
			{
			case ARRIBA:
				aumento_fila = -1;
				aumento_col = 0;
				break;
			case ABAJO:
				aumento_fila = 1;
				aumento_col = 0;
				break;
			case IZQUIERDA:
				aumento_fila = 0;
				aumento_col = -1;
				break;
			case DERECHA:
				aumento_fila = 0;
				aumento_col = 1;
				break;
		}
	}
	else if (giro == 'i')
	 {
		/* Nos fijamos la direccin del gusano... */
		switch (Gusano->direccion)
		{
			case ARRIBA:
				aumento_fila = 0;
				aumento_col = -1;

				Gusano->direccion = IZQUIERDA;
				break;
			case ABAJO:
				aumento_fila = 0;
				aumento_col = 1;

				Gusano->direccion = DERECHA;
				break;
			case IZQUIERDA:
				aumento_fila = 1;
				aumento_col = 0;

				Gusano->direccion = ABAJO;
				break;
			case DERECHA:
				aumento_fila = -1;
				aumento_col = 0;

				Gusano->direccion = ARRIBA;
				break;
		}
	}
	else if (giro == 'd')
	{
		/* Nos fijamos la direccin del gusano... */
		switch (Gusano->direccion)
		{
			case ARRIBA:
				aumento_fila = 0;
				aumento_col = 1;

				Gusano->direccion = DERECHA;
				break;
			case ABAJO:
				aumento_fila = 0;
				aumento_col = -1;

				Gusano->direccion = IZQUIERDA;
				break;
			case IZQUIERDA:
				aumento_fila = -1;
				aumento_col = 0;

				Gusano->direccion = ARRIBA;
				break;
			case DERECHA:
				aumento_fila = 1;
				aumento_col = 0;

				Gusano->direccion = ABAJO;
				break;
		}
	 }

	/* Vemos si el gusano esta creciendo o no */
	if (Creciendo == 1)
	{
		temp = NuevoNodo();
		temp->columna = cabeza->columna + aumento_col;
		temp->fila = cabeza->fila + aumento_fila;
		temp->siguiente = cabeza;

		cabeza = temp;

		gotoxy(cabeza->columna, cabeza->fila);
		cprintf("%c", 219);

		VecesQueDebeCrecer--;
		if (VecesQueDebeCrecer == 0)
			Creciendo = 0;
	 }
	 else
	 {
			/* Creamos una lista auxiliar con las direcciones de los nodos de la lista
		       del gusano, pero desde la cola hacia la cabeza. Esto es necesario
			   para mover al gusano. */
			temp = cabeza;
			while (temp != NULL)
			{
				 temp2 = (aux *)malloc(sizeof(aux));
				 temp2->info = temp;
				 temp2->siguiente = lista_aux;

				 lista_aux = temp2;

				 temp = temp->siguiente;
			}

			/* Cambiamos las posiciones */
			gotoxy(lista_aux->info->columna, lista_aux->info->fila); /* Esta es la cola */
			cprintf(" ");

			temp2 = lista_aux;
			while (temp2->siguiente != NULL)
			{
				 temp2->info->columna = temp2->siguiente->info->columna;
				 temp2->info->fila = temp2->siguiente->info->fila;

				 temp2 = temp2->siguiente;
			}
			
			/* Liberamos la memoria ocupada por la lista auxiliar */
			while (lista_aux != NULL)
			{
				temp2 = lista_aux;
				lista_aux = lista_aux->siguiente;
				
				free(temp2);
			}

			/* Adelantamos una posicion la cabeza del gusano */
			cabeza->columna += aumento_col;
			cabeza->fila += aumento_fila;

			gotoxy(cabeza->columna, cabeza->fila);
			cprintf("%c", 219);
		}

	 /* Chequemos si el gusano comio una ficha, o choc un obstculo */
	 /* La funcion QueHay devuelve un 0 si no hay nada, un uno si hay un obstaculo, y
	 un 2 si hay un ficha */
	 Resultado_QueHay = QueHay(obstaculos, *fichas, cabeza, &FichaSobreGusano, cabeza->columna, cabeza->fila);
	 if (Resultado_QueHay == 1)
			return 1;
	 else if (Resultado_QueHay == 2)
	 {
		/* Como comi una ficha, aumentamos en 5 su longitud */
		Gusano->longitud += CRECIMIENTO;
		Gusano->puntaje += PUNTAJECOMERFICHA;
		Gusano->fichas_comidas++;	 		
	 		
		/* Mostramos el puntaje */
		Datos(*Gusano);
	 		
		/* Hacemos un ruidito */
        sound(440); delay(20);	nosound();
	 		
		/* Eliminamos la 'tapa' de la pila */
  	 	EliminarPrimerFicha(fichas);
  	 	
  	 	/* Nos fijamos si la prxima ficha parecer sobre el gusano en caso de ser as esperamos a que el 
           gusano abandone la posicin y recien cuando ya no est encima mostramos la ficha */
		FichaSobreGusano = MostrarSiguienteFicha(*fichas, cabeza);

		Creciendo = 1;
		VecesQueDebeCrecer += 5;
	 }

	 *p = cabeza;
	 return 0;
}


