/*
 * Eufruncio - El gusanito comefichas
 * Copyright (C) 2003-2004 Sebastin Galiano, Milton Pividori, Csar Sandrigo
 * 
 * This file is part of Eufruncio.
 * 
 * Eufruncio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Eufruncio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Eufruncio; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/************************************************************************************************
  Definimos los tipos que utilizaremos en este archivo
************************************************************************************************/

typedef struct obstaculos
{
  char columna;
  struct obstaculos *siguiente;
}obs;


typedef struct fichas
{
  char columna;
  char fila;
  struct fichas *siguiente;
}ficha;



/************************************************************************************************
  Libera la memoria utilizada por la pila que contiene los datos de las fichas
************************************************************************************************/

void LiberarFichas(ficha **fichas)
{
  ficha *aux1 = *fichas;
  ficha *aux2;
	
  while (aux1 != NULL)
  {
    aux2 = aux1;
	aux1 = aux1->siguiente;
	
	free(aux2);
  }
	
  *fichas = aux1;
}



/************************************************************************************************
  Libera la memoria utilizada por el vector de listas que administra los obstculos
************************************************************************************************/

void LiberarObstaculos(obs *obstaculos[])
{	
  obs *aux1;
  obs *temp1;
  char i = 0;
	
  while (i <= 44)
  {
	aux1 = obstaculos[i];
		
	while (aux1 != NULL)
	{
	  temp1 = aux1;
	  aux1 = aux1->siguiente;
		
	  free(temp1);
	}
		
	i++;
  }
}



/************************************************************************************************
  Agrega un nuevo nodo de la lista que almacena las fichas
************************************************************************************************/

ficha *NuevaFicha()
{
  ficha *p;
  p = (ficha *)malloc(sizeof(ficha));   
  
  return p;
}



/************************************************************************************************
  Agrega un nuevo nodo de la lista que almacena los obstculos
************************************************************************************************/

obs *NuevoObstaculo()
{
  obs *p;
  p = (obs *)malloc(sizeof(obs));
  
  return p;
}



/************************************************************************************************
  Agrega las fichas a la pila 
************************************************************************************************/

void AgregarFicha(ficha **fichas, ficha **auxiliar, char columna, char fila)
{
   /* Vamos haciendo una copia de las fichas en auxiliar */
   ficha *aux;
   ficha *aux2;
   
   aux = NuevaFicha();
   aux->columna = columna;
   aux->fila = fila;
   aux->siguiente = *fichas;
   
   aux2 = NuevaFicha();
   aux2->columna = columna;
   aux2->fila = fila;
   aux2->siguiente = *auxiliar;


   *fichas = aux;
   *auxiliar = aux2;
}



/************************************************************************************************
  Ordena los obstaculos en una lista 
************************************************************************************************/

void AgregarObstaculo(obs *obstaculos[], char columna, char fila)
{
   /* Punteros auxiliares */
   obs *aux = obstaculos[fila];
   obs *aux2;
   
   /* Si nos enviamos un puntero NULL, significa que esa componente del vector
      es nula, y asi creamos una nueva en dicha posicion */
   if (aux == NULL)
   {
      aux = NuevoObstaculo();
      aux->columna = columna;
      aux->siguiente = NULL;
   }
   else
   {
      /* Suponemos que en el archivo de configuracin no hay dos lineas que
         contengan un obstaculo con las mismas coordenadas */
      while ((aux->siguiente != NULL) && (aux->siguiente->columna < columna))
         aux = aux->siguiente;

      /* Creamos el nuevo elemento */
      aux2 = NuevoObstaculo();
      aux2->columna = columna;
   
      /* Chequeamos por qu salimos del while anterior */
      if (aux->siguiente == NULL)
      {
         /* Que 'aux->siguiente' sea igual a NULL implica que aux es el ltimo
            elemento de la lista, pero debemos verificar que no sea el nico
            (el primero) */
         if (aux->columna < columna)
         {
            aux2->siguiente = NULL;
            aux->siguiente = aux2;
         }
         else
         {
            /* Si entramos aqu, es porque solo hay un elemento en la lista y
               el nuevo obstaculo debe ir delante del existente, porque es
               menor. */
            
            aux2->siguiente = aux;
            aux = aux2; /* Para que la sentencia de abajo, 'obstaculos[fila]=aux'
                           sea correcta, es decir, que 'obstaculos[fila]' apunte
                           al primer elemento de la lista. */
         }
      }
      else
      {
         aux2->siguiente = aux->siguiente;
         
         aux->siguiente = aux2;
      }
   }
   
   obstaculos[fila] = aux;
}



/************************************************************************************************
  Funcin principal del archivo en la que se lee el archivo de configuracin y se realiza todas 
  las operaciones correspondientes: se almacena en memoria las fickhas y los obstculos 
************************************************************************************************/

void CargarConfiguracion(obs *obstaculos[], ficha **fichas, ficha **auxiliar)
{
   /* Vamos a hacer una aclaracion importante antes de continuar:
   Nosotros hemos utilizado la funcin 'textmode' para emular una pantalla de
   80 columnas por 50 filas. Por qu? Porque si no lo hacemos de esta manera,
   el gusano ser mas fino cuando vaya en direccin vertical, y creemos que
   este detalle no sera de agrado para el jugador... =)
   Como hemos utilizado bordes que delimitan el tablero de juego, y hemos
   utilizado filas para colocar el puntaje del jugador y otros datos, el rea
   real de juego (es decir, donde se mueve el gusano) es de 78 columnas por 45
   filas. Debemos tener en cuenta estos rangos al editar el archivo
   'config.txt'. */

   FILE *arc;

   /* No hace falta que usemos un int para columna y fila, porque los valores
      no superarn al 78 */
   char letra, columna, fila, aux;

   arc = fopen("config.txt", "r");
   if (arc == NULL)
   {
      clrscr();
      cprintf("No fue posible abrir el archivo 'config.txt'. Saliendo...");
      exit(1);
   }

   /* Si no hubo errores, comenzamos a leer el archivo */
   while (!feof(arc) && !ferror(arc))
   {
      letra = fgetc(arc); /* Tomamos la primer letra, que ser una 'o' o 'f' */
      fgetc(arc); /* Salteamos la coma */
      columna = fgetc(arc); /* Tomamos la columna, pero... */

      /* ...Si la la columna es nmero de dos cifras, con la sentencia anterior
         solo estaramos tomando la decena. Por eso usamos 'aux'. Si el cdigo
         ASCII de aux esta entre el 48 y el 57, significa que la columna es un
         nmero de dos dgitos. Si el cdigo es 44, significa que es la coma, y
         por lo tanto la columna es un nmero entre el 0 y el 9, y deberamos
         comenzar a leer la fila. */
      aux = fgetc(arc);

      if ((aux >= 48) && (aux <= 57))
      {
         /* Por qu 'columna - 48'? Porque si al cdigo ASCII de los nmero le
            restamos 48, nos da el nmero que representa dicho valor ASCII. */
         columna = ((columna - 48) * 10) + (aux - 48);

         /* Leemos la coma. Estamos seguros de esto, porque la columna, como la
            fila, no ser un valor de mas de dos dgitos */
         fgetc(arc);
      }
      else
         columna = columna - 48;

      /* Todo lo anterior se repite para la fila... */
      fila = fgetc(arc);
      aux = fgetc(arc);
      if ((aux >= 48) && (aux <= 57))
      {
         fila = ((fila - 48) * 10) + (aux - 48);
         fgetc(arc); /* Pasamos el retorno de linea por alto */
      }
      else
         fila = fila - 48;

      /* Ahora comenzamos a chequear que es lo que se ha leido */
      /* Si la letra es la O mayscula (obstaculo)... */
      if (toupper(letra) == 79)
      	/* El -1 es porque obstaculos ser un vector, y su primer componente es
      		el 0 */
         AgregarObstaculo(obstaculos, columna, fila - 1);
      else if (toupper(letra) == 70)
         AgregarFicha(fichas, auxiliar, columna, fila);
   }

   if (ferror(arc))
   {
      clrscr();
      cprintf("Error al intentar leer en el fichero. Saliendo...");
      exit(1);
   }
}

