/*
 * Eufruncio - El gusanito comefichas
 * Copyright (C) 2003-2004 Sebastin Galiano, Milton Pividori, Csar Sandrigo
 * 
 * This file is part of Eufruncio.
 * 
 * Eufruncio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Eufruncio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Eufruncio; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/************************************************************************************************
	 Definimos los tipos que utilizaremos
************************************************************************************************/

typedef struct repeticion
{
   char cantidad_mov;
   char tecla;
   struct repeticion *siguiente;
}rep;



/************************************************************************************************
	 Reserva dinmicamente memoria para guardar los movimientos que el gusano realiza
************************************************************************************************/

rep *NuevoRep()
{
   rep *aux;
   aux = (rep *)malloc(sizeof(rep));
   
   return aux;
}



/************************************************************************************************
	Liberamos de la memoria que reservarmos con la funcin anterior
************************************************************************************************/

/* Funcion para liberar memoria */
void LiberarRep(rep **inicio_rep, rep **fin_rep)
{

	rep *aux1 = *inicio_rep;
	rep *aux2 = *fin_rep;
	rep *temp;
	
	while ((aux1 != NULL) && (aux2 != NULL) && (aux1 != aux2))
	{
		temp = aux1;
		aux1 = aux1->siguiente;
		
		free(temp);
	}
	
	if ((aux1 != NULL) && (aux2 != NULL))
	{
		/* Este cdigo deja en NULL a aux1 y aux2, liberando la memoria ocupada
  		   por ambos */
		free(aux1);
		aux2 = aux1 = NULL;
	}

	*inicio_rep = aux1;
	*fin_rep = aux2;
}



/************************************************************************************************
	Agregamos una tecla a la cola que simula los movimientos
************************************************************************************************/

void AgregarTecla(rep **inicio, rep **fin, char tecla, char *cantidad_mov)
{
   rep *aux1 = *inicio;
   rep *aux2 = *fin;
   rep *temp;
   
   temp = NuevoRep();
   temp->cantidad_mov = *cantidad_mov;
   temp->tecla = tecla;
   temp->siguiente = NULL;
   
   if (aux1 == NULL)
   {
      aux1 = temp;
      aux2 = temp;
   }
   else if (aux1 == aux2)
   {
      aux1->siguiente = temp;
      aux2 = temp;
   }
   else
   {
      aux2->siguiente = temp;
      aux2 = temp;
   }
      
   *inicio = aux1;
   *fin = aux2;
   *cantidad_mov = 0;
}



/************************************************************************************************
	Mientras repetimos
************************************************************************************************/

void Repitiendo(rep **inicio_rep, rep **fin_rep)
{
   rep *aux1 = *inicio_rep;
   rep *aux2 = *fin_rep;
   rep *temp;
   
   if (aux1->cantidad_mov != 0)
      aux1->cantidad_mov--;
   else
   {
      temp = aux1;
      aux1 = aux1->siguiente;

      free(temp);
   }

   *inicio_rep = aux1;
   *fin_rep = aux2;
}



/************************************************************************************************
	Rutina principal que repite la jugada
************************************************************************************************/

void Repetir(rep **inicio_rep, rep **fin_rep, obs *obstaculos[], ficha **fichas, unsigned int Velocidad)
{
	/* Variables relacionadas con el gusano */
	gusano *cabeza = NULL;
	ficha *aux_fichas = *fichas;	
    char tecla, i, giro, termina = 0, salir = 0;
	datos Eufruncio;

	/* Utilizadas para repetir */
	rep *inicio = *inicio_rep;
	rep *fin = *fin_rep;

	 /* Inicializamos la estructura con datos del gusano */
	 Eufruncio.longitud=LONGITUDINICIAL;
	 Eufruncio.direccion = ARRIBA;
	 Eufruncio.puntaje = 0;
	 Eufruncio.fichas_comidas = 0;

	 /* Inicializamos el tablero */
	 textbackground(COLORFONDO); clrscr();
	 DibujarParedes(COLORPARED);
	 textcolor(COLORPARED); Centrar(50, "Repitiendo la jugada...");
	 textbackground(COLORFONDO);
	 CrearGusano(&cabeza);
	 VisualizarObstaculos(obstaculos);

 	 /* Ahora visualiamos la primer ficha */
	 MostrarSiguienteFicha(aux_fichas, cabeza);

	 /* Mostramos el ttulo del juego */
	 textcolor(COLORTITULO); Centrar(1, ">>> EUFRUNCIO, EL GUSANITO COMEFICHAS <<<");

	 Datos(Eufruncio);

	 /* Seteamos el color del gusano */
	 textcolor(COLORGUSANO);

	 /* Este cdigo hace que el gusano se mueva */
	 while (!termina && !salir)
	 {
			/* Hacemos la pausa */
			delay(Velocidad);

			if (inicio != NULL)
			{
				 if (inicio->cantidad_mov == 0)
				 {
				    tecla = inicio->tecla;
				    Repitiendo(&inicio, &fin);

                 if (tecla == ESCAPE)
                 {
		           salir = Confirmacion("Desea terminar la repeticin? (S/N)");
		           if (!salir)
                   {
                     VisualizarObstaculos(obstaculos);
                     VisualizarFicha(aux_fichas->columna, aux_fichas->fila);                
                   }
                 }

   				 if (!tecla)
				    {
				       tecla = inicio->tecla;
				       Repitiendo(&inicio, &fin);

						 /* Le damos direccin al gusano */
						 switch (tecla)
						 {
						    case TECLAARRIBA:
								 if ((Eufruncio.direccion == ARRIBA) || (Eufruncio.direccion == ABAJO))
									 giro = ' ';
								 if (Eufruncio.direccion == IZQUIERDA)
 									 giro = 'd';
								 else if (Eufruncio.direccion == DERECHA)
									 giro = 'i';
								 break;

							 case TECLAABAJO:
								 if ((Eufruncio.direccion == ARRIBA) || (Eufruncio.direccion == ABAJO))
									 giro = ' ';
								 if (Eufruncio.direccion == IZQUIERDA)
									 giro = 'i';
	 							 else if (Eufruncio.direccion == DERECHA)
									 giro = 'd';;
								 break;

							 case TECLAIZQUIERDA:
								 if ((Eufruncio.direccion == DERECHA) || (Eufruncio.direccion == IZQUIERDA))
									 giro = ' ';
								 if (Eufruncio.direccion == ARRIBA)
									 giro = 'i';
								 else if (Eufruncio.direccion == ABAJO)
									 giro = 'd';;
								 break;

							 case TECLADERECHA:
								 if ((Eufruncio.direccion == DERECHA) || (Eufruncio.direccion == IZQUIERDA))
									 giro = ' ';
								 if (Eufruncio.direccion == ARRIBA)
									 giro = 'd';
								 else if (Eufruncio.direccion == ABAJO)
									 giro = 'i';
								 break;
						}
					  }
				     else
						  giro = ' ';
			      }
			      else
				      giro = ' ';
		      }
		      else
		      	giro = ' ';

		   if (inicio != NULL)
		   	Repitiendo(&inicio, &fin);

			termina = MoverGusano(&cabeza, giro, &Eufruncio, obstaculos, &aux_fichas);
	 }


  	 /* Mostramos en la barra de estado */
  	 gotoxy(1,50);
	 if (termina == 1)
	   Centrar(50, "Perdiste!! Nabo.....");
	 else if (termina == 2)
	   Centrar(50, "Felicitaciones, te morfaste todas las fichas...");

	 /* Liberamos la memoria */
     LiberarGusano(&cabeza);
     LiberarFichas(&aux_fichas);
     *fichas = aux_fichas;
	 *inicio_rep = inicio;
	 *fin_rep = fin;

	 /* Mostramos la ventana de finalizacin del proceso */
	 Centrar(50, "                       ");
     textcolor(COLORDIALOGOS); textbackground(COLORFONDODIALOGOS);
     Centrar(23, "");
     Centrar(24, "                                                   ");
     Centrar(25, "    Repeticin terminada. Presione una tecla...    ");
     Centrar(26, "                                                   ");
     Centrar(27, "");

     /* Esperamos que se presione una tecla */
	 getch();
	 
     /* Vaciamos el buffer del teclado */
     fflush(stdin);   

  }

